/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.apprtc.bean;

import es.rickyepoderi.apprtc.ejb.RoomBean;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;
import java.util.Random;
import javax.inject.Named;
import javax.enterprise.context.RequestScoped;
import javax.faces.context.FacesContext;
import javax.json.Json;
import javax.json.stream.JsonGenerator;
import javax.servlet.http.HttpServletRequest;

/**
 * <p>Tricky JSF request bean that handles with all the things tha
 * the python does in the apprtc google application. It generates all the
 * variables and so on.</p>
 * 
 * <p>TODO: Not all the parameters of the query string are managed.</p>
 * 
 * @author ricky
 */
@Named(value = "appRTCBean")
@RequestScoped
public class AppRTCBean {

    /**
     * The room key (string of 8 digits randomly generated)
     */
    private String roomKey = null;
    
    /**
     * The username (string of 8 digits randomly generated)
     */
    private String username = null;
    
    /**
     * 0 means there is no other partner in the room (exchange should not
     * be started) and 1 the other partner is waiting (start the exchange).
     */
    private int initiator = 0;
    
    /**
     * The rom bean singleton.
     * TODO: Here the EJB worked well but no in the websocket endpoint, it
     *       seems as it cannot be injected in endpoints. So normal singleton.
     */
    RoomBean roomBean = RoomBean.getRoomBean();
    
    /**
     * Empty constructor
     */
    public AppRTCBean() {
        System.err.println("AppRTCBean");
    }
    
    /**
     * Helper to create the random room keys and usernames
     * @param length The length of the string
     * @return The generated string
     */
    private String generateRandom(int length) {
        StringBuilder sb = new StringBuilder();
        Random random = new Random();
        for (int i=0; i < length; i++) {
            sb.append(random.nextInt(10));
        }
        return sb.toString();
    }
    
    /**
     * Returns the key. The key can be a parameter "r" in the URL or it is
     * generated randomly.
     * @return 
     */
    public String getRoomKey() {
        if (roomKey == null) {
            HttpServletRequest request = (HttpServletRequest) FacesContext
                    .getCurrentInstance().getExternalContext().getRequest();
            roomKey = request.getParameter("r");
            if (roomKey == null) {
                roomKey = generateRandom(8);
            }
        }
        return roomKey;
    }
    
    /**
     * returns the room link depending in the room key generated.
     * @return The room link
     */
    public String getRoomLink() {
        HttpServletRequest request = (HttpServletRequest) FacesContext
                    .getCurrentInstance().getExternalContext().getRequest();
        String url = request.getRequestURL().toString();
        if (url.endsWith("full.xhtml")) {
            url = url.substring(0, url.length() - 10).concat("index.xhtml");
        }
        StringBuilder sb = new StringBuilder(url);
        sb.append("?r=").append(getRoomKey());
        for (Map.Entry<String, String[]> entry : request.getParameterMap().entrySet()) {
            if (!entry.getKey().equals("r")) {
                for (String v: entry.getValue()) {
                    try {
                        sb.append("&").append(entry.getKey()).append("=").append(URLEncoder.encode(v, "UTF-8"));
                    } catch (UnsupportedEncodingException e) {}
                }
            }
        }
        return sb.toString();
    }
    
    /**
     * Generates a random username and return it.
     * @return The generated username
     */
    public String getUsername() {
        if (username == null) {
            username = generateRandom(8);
        }
        return username;
    }
    
    /**
     * It checks if the other user in the room is already waiting and put 
     * initiator to 0 or 1.
     * @return 1 or 0 depending the partner in the is the room is already waiting or not.
     */
    public int getInitiator() {
        initiator = roomBean.isTheOtherMemberWaiting(getRoomKey(), getUsername())? 1 : 0;
        return initiator;
    }
    
    /**
     * It returns the JSON configuration for the PC_CONFIG. The PC_CONFIG seems to
     * have the information for stun and turn servers. It manager three 
     * parameters: "ts" -> trurn server; "tp" -> turn password; "ss" -> stun server.
     * 
     * TODO: For the moment the request parameters are not used and only default
     *       configuration is returned.
     * @return The JSON string for PC CONFIG
     */
    public String getPcConfig() {
        HttpServletRequest request = (HttpServletRequest) FacesContext
                    .getCurrentInstance().getExternalContext().getRequest();
        String turnServer = request.getParameter("ts");
        String turnServerPwd = request.getParameter("tp");
        String stunServer = request.getParameter("ss");
        // for the moment just return the default stun
        StringWriter writer = new StringWriter();
        try (JsonGenerator gen = Json.createGenerator(writer)) {
            gen.writeStartObject()
                .writeStartArray("iceServers")
                    .writeStartObject()
                        .write("url", "stun:stun.l.google.com:19302")
                    .writeEnd()
                .writeEnd()
            .writeEnd();
        }
        return writer.toString();
    }
    
    /**
     * It returns the JSON for the PC CONSTRAINTS part. It manages a "compat"
     * parameter.
     * @return The JSON for the PC CONSTRAINTS.
     */
    public String getPcConstraints() {
        HttpServletRequest request = (HttpServletRequest) FacesContext
                    .getCurrentInstance().getExternalContext().getRequest();
        String compatStr = request.getParameter("compat");
        boolean compat = true;
        if (compatStr != null) {
            compat = Boolean.parseBoolean(compatStr);
        }
        StringWriter writer = new StringWriter();
        try (JsonGenerator gen = Json.createGenerator(writer)) {
            gen.writeStartObject().writeStartArray("optional");
            if (compat) {
                gen.writeStartObject()
                    .write("DtlsSrtpKeyAgreement", true)
                .writeEnd();
            }
            gen.writeEnd()
                    .writeEnd();
        }
        return writer.toString();
    }
    
    /**
     * it Returns the OFFER CONSTRAINTS.
     * @return The JSON for the OFFER CONSTRAINTS.
     */
    public String getOfferConstraints() {
        StringWriter writer = new StringWriter();
        try (JsonGenerator gen = Json.createGenerator(writer)) {
            gen.writeStartObject()
                .writeStartObject("mandatory")
                .writeEnd()
                .writeStartArray("optional")
                .writeEnd()
            .writeEnd();
        }
        return writer.toString();
    }
    
    /**
     * The MEDIA CONSTRAINTS manage three parameters: "media" (audio, video or
     * both), min_re and max_re (these two arguments seem to be for video 
     * resolution).
     * 
     * TODO: For the moment the default constraints are returned, the parameters
     *       are not read.
     * @return 
     */
    public String getMediaConstraints() {
        // for the moment default audio and video
        StringWriter writer = new StringWriter();
        try (JsonGenerator gen = Json.createGenerator(writer)) {
            gen.writeStartObject()
                .write("audio", true)
                .writeStartObject("video")
                    .writeStartArray("optional")
                    .writeEnd()
                    .writeStartObject("mandatory")
                    .writeEnd()
                .writeEnd()
            .writeEnd();
        }
        return writer.toString();
    }
    
    /**
     * It returns the same turn URL that the python apprtc returns.
     * @return The TURN url
     */
    public String getTurnUrl() {
        return new StringBuilder()
                .append("https://computeengineondemand.appspot.com/")
                .append("turn?")
                .append("username=").append(getUsername())
                .append("&key=4080218913")
                .toString();
    }
    
    /**
     * Stereo os not stereo sound. It is managed by the "stereo" request
     * parameter.
     * @return true or false.
     */
    public boolean getStereo() {
        boolean stereo = false;
        HttpServletRequest request = (HttpServletRequest) FacesContext
                    .getCurrentInstance().getExternalContext().getRequest();
        String stereoStr = request.getParameter("stereo");
        if (stereoStr != null) {
            stereo = Boolean.parseBoolean(stereoStr);
        }
        return stereo;
    }
    
    public boolean isTheRoomFull() {
        return roomBean.isTheRoomFull(getRoomKey());
    }
}
