/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.apprtc.server;

import es.rickyepoderi.apprtc.ejb.RoomBean;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import javax.json.Json;
import javax.json.JsonObject;
import javax.json.JsonReader;
import javax.json.JsonWriter;
import javax.validation.constraints.NotNull;
import javax.websocket.OnClose;
import javax.websocket.OnMessage;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;

/**
 *
 * <p>WebSocket endpoint that handles with the clients messages. It manages 
 * three types of messages (all of them are JSON messages):</p>
 * 
 * <ul>
 *   <li>join: A new user is joining to the room (type, username and roomKey).</p>
 *   <li>bye: The user is leaving the room.</li>
 *   <li>other: Other messages which are re-sent to the partner in the room
 *       (messages like "offer", "candidate").</li>
 * </ul>
 * 
 * @author ricky
 */
@ServerEndpoint("/roomendpoint")
public class RoomEndpoint {
    
    /**
     * The room bean singleton.
     */
    RoomBean roomBean = RoomBean.getRoomBean();

    /**
     * The onclose is used to send "bye" to the partner in the room.
     * @param session The session who is closing
     */
    @OnClose
    public void onClose (Session session) {
        System.err.println("Closing peer=" + session);
        roomBean.disconnect(session);
    }
    
    /**
     * The real method that handles with the messages.
     * @param message The message of the client
     * @param session The client session
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        System.err.println(message);
        try (JsonReader reader = Json.createReader(new StringReader(message))) {
            JsonObject data = (JsonObject) reader.read();
            String type = data.getString("type");
            switch (type) {
                case "join":
                    // connect the user to the room
                    String roomKey = data.getString("roomKey");
                    String username = data.getString("username");
                    roomBean.connect(roomKey, username, session);
                    break;
                case "bye":
                    // disconnect the user
                    roomBean.disconnect(session);
                    break;
                default:
                    // we have to deal with the message and send it to the other user
                    roomBean.sendToTheOther(session, data);
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Helper to close a websocket session.
     * @param s The session to close
     */
    static public void closeSession(Session s) {
        try {
            if (s != null) {
                s.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Helper to send a a JSON message to another session.
     * @param s The session to send the message to
     * @param data The data to send
     */
    static public void send(@NotNull Session s, @NotNull JsonObject data) {
        StringWriter stWriter = new StringWriter();
        try (JsonWriter jsonWriter = Json.createWriter(stWriter)) {
            jsonWriter.writeObject(data);
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.err.println("Sending: " + stWriter.toString());
        try {
            s.getBasicRemote().sendObject(stWriter.toString());
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
}
